/***************************************************************************
 *   Copyright (C) 2008 by Rachmaninov,,,   							   *
 *   rachmaninov2@gmail.com												   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <QStack>
#include <QFile>

#include "odthtml.h"

OdtHtml::OdtHtml(const QString& filename) : sw(&html) {
	this->filename = filename;
}

OdtHtml::~OdtHtml() {
	// Libera los elementos del mapa
	QList<TextFormat*> values = mapStyles.values();
	for (int i=0; i<values.size(); i++) {
		delete values.at(i);
	}
}

void OdtHtml::convert() {

	QFile file(filename);
	if (!file.open(QIODevice::ReadOnly)) {
		throw "Error al abrir el archivo.";
	} 
	
	QByteArray content = file.readAll();

	QDomDocument doc;
	int line, col;
	QString errorMessage;

	if (!doc.setContent(content, &errorMessage, &line, &col)) {
		QString message = QObject::tr("Error en (%1,%2): %3").arg(line).arg(col).arg(errorMessage);
		throw message;
	}

	QDomElement rootElement = doc.documentElement();

	processStyles(rootElement);
	processOfficeText(rootElement);

	file.close();
}

void OdtHtml::processStyles(const QDomElement& rootElement) {
	QDomNodeList list = rootElement.elementsByTagName("office:automatic-styles");
	if (list.length() > 0) {
		QDomNode process = list.item(0);
		QDomNodeList items = process.childNodes();
		for (uint i=0; i<items.length(); ++i) {
			QDomNode node = items.item(i);
			processStyleNode(node);
		}
	}
}

void OdtHtml::processStyleNode(const QDomNode& node) {
	if (node.isElement() && (node.nodeName() == "style:style")) {
		QDomElement element = node.toElement();
		QString family = element.attribute("style:family", QString::null);
		QString name = element.attribute("style:name", QString::null);
		if (family == "text") {
			QDomNode child = element.firstChild();
			if ((!child.isNull()) && child.isElement() && 
				(child.nodeName() == "style:text-properties")) {
				QDomElement textPropertiesNode = child.toElement();
				int style;

				// Bold

				QString fontWeight = textPropertiesNode.attribute("fo:font-weight",
					QString::null);
				if (fontWeight == "bold") {
					style = style | TextFormat::Bold;
				}

				// Estilo

				QString fontStyle = textPropertiesNode.attribute("fo:font-style",
					QString::null);
				if (fontStyle == "italic") {
					style = style | TextFormat::Italic;
				}

				// Underline

				QString textUnderline = textPropertiesNode.attribute(
					"style:text-underline-style", QString::null);
				if (textUnderline == "solid") {
					style = style | TextFormat::Underline;
				}

				TextFormat* textStyle = new TextFormat();
				textStyle->style = style;
				mapStyles.insert(name, textStyle);
			}
		}		
	}
}

void OdtHtml::processOfficeText(const QDomElement& rootElement) {
	QDomNodeList list = rootElement.elementsByTagName("office:text");
	if (list.length() > 0) {
		QDomNode process = list.item(0);
		QDomNodeList items = process.childNodes();
		for (uint i=0; i<items.length(); ++i) {
			QDomNode node = items.item(i);
			processOfficeTextChild(node);
		}
	}
}

void OdtHtml::processOfficeTextChild(const QDomNode& node) { 
	if (node.isText()) {
		sw.writeCharacters(node.toText().data());
	} else if (node.isElement()) {
		QDomElement element = node.toElement();
		QString tagName;
		QXmlStreamAttributes attrs;
		QString tagNameOdt = element.tagName();
		QStack<XmlTag*> stack;
		if (tagNameOdt == "text:h") {
			QDomAttr attr = element.attributeNode("text:outline-level");
			if (!attr.isNull()) {
				XmlTag* xmltag = new XmlTag();
				xmltag->name = "h" + attr.value();
				stack.push(xmltag);
			}
		} else if (tagNameOdt == "text:p") {
			XmlTag* xmltag = new XmlTag();
			xmltag->name = "p";
			stack.push(xmltag);	
		} else if (tagNameOdt == "text:span") {
			QString styleName = element.attribute("text:style-name", QString::null);
			
			if (mapStyles.contains(styleName)) {
				TextFormat* textStyle = mapStyles.take(styleName);
				if (textStyle->style & TextFormat::Bold) {
					XmlTag* xmltag = new XmlTag();
					xmltag->name = "b";
					stack.push(xmltag);
				}

				if (textStyle->style & TextFormat::Italic) {
					XmlTag* xmltag = new XmlTag();
					xmltag->name = "i";
					stack.push(xmltag);
				}	

				if (textStyle->style & TextFormat::Underline) {
					XmlTag* xmltag = new XmlTag();
					xmltag->name = "u";
					stack.push(xmltag);
				}
			}			
		} else if (tagNameOdt == "text:a") {
			XmlTag* xmltag = new XmlTag();
			xmltag->name = "a";
			QDomAttr href = element.attributeNode("xlink:type");
			if (!href.isNull()) {
				xmltag->attrs.append("href", href.value());
			}
			stack.push(xmltag);			
		}

		int tagsCount = stack.size();
	
		// Escribe la apertura de los tags

		while (!stack.isEmpty()) {
			XmlTag* xmltag = stack.pop();
			sw.writeStartElement(xmltag->name);
			if (!xmltag->attrs.isEmpty()) {
				sw.writeAttributes(xmltag->attrs);
			}
			delete xmltag;
		}

		// Procesa el contenido anidado

		QDomNodeList childs = node.childNodes();
		for (uint i=0; i<childs.length(); ++i) {
			QDomNode node = childs.item(i);
			processOfficeTextChild(node);
		}

		// Escribe el cierre de los tags abiertos

		for (int i=0; i<tagsCount; ++i) {
			sw.writeEndElement();
		}
	}
}
